@@ -1,33 +1,7 @@
-(******************************************************************************
- *                             Bin-prot                                       *
- *                                                                            *
- * Copyright (C) 2008- Jane Street Holding, LLC                               *
- *    Contact: opensource@janestreet.com                                      *
- *    WWW: http://www.janestreet.com/ocaml                                    *
- *    Author: Markus Mottl                                                    *
- *                                                                            *
- * This library is free software; you can redistribute it and/or              *
- * modify it under the terms of the GNU Lesser General Public                 *
- * License as published by the Free Software Foundation; either               *
- * version 2 of the License, or (at your option) any later version.           *
- *                                                                            *
- * This library is distributed in the hope that it will be useful,            *
- * but WITHOUT ANY WARRANTY; without even the implied warranty of             *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU          *
- * Lesser General Public License for more details.                            *
- *                                                                            *
- * You should have received a copy of the GNU Lesser General Public           *
- * License along with this library; if not, write to the Free Software        *
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  *
- *                                                                            *
- ******************************************************************************)
-
 (* Read_ml: reading values from the binary protocol using (mostly) OCaml. *)
 
 #include "int_codes.mlh"
 
-open Bigarray
-
 open Common
 
 external init : unit -> unit = "bin_prot_read_ml_init_stub"
@@ -385,7 +359,7 @@ let bin_read_triple bin_read_a bin_read_b bin_read_c buf ~pos_ref =
   let c = bin_read_c buf ~pos_ref in
   a, b, c
 
-let rec bin_read_n_rev_list bin_read_el buf ~pos_ref len =
+let bin_read_n_rev_list bin_read_el buf ~pos_ref len =
   let rec loop n acc =
     if n = 0 then acc
     else loop (n - 1) (bin_read_el buf ~pos_ref :: acc)
@@ -398,7 +372,7 @@ let bin_read_list bin_read_el buf ~pos_ref =
   List.rev rev_lst
 
 #ifndef ARCH_SIXTYFOUR
-let dummy_float_buf = Array1.create char c_layout 8
+let dummy_float_buf = create_buf 8
 let () = ignore (Write_ml.bin_write_float dummy_float_buf ~pos:0 3.1)
 let max_array_length_2 = Sys.max_array_length / 2
 #endif
