@@ -1,30 +1,5 @@
-(******************************************************************************
- *                             Bin-prot                                       *
- *                                                                            *
- * Copyright (C) 2008- Jane Street Holding, LLC                               *
- *    Contact: opensource@janestreet.com                                      *
- *    WWW: http://www.janestreet.com/ocaml                                    *
- *    Author: Markus Mottl                                                    *
- *                                                                            *
- * This library is free software; you can redistribute it and/or              *
- * modify it under the terms of the GNU Lesser General Public                 *
- * License as published by the Free Software Foundation; either               *
- * version 2 of the License, or (at your option) any later version.           *
- *                                                                            *
- * This library is distributed in the hope that it will be useful,            *
- * but WITHOUT ANY WARRANTY; without even the implied warranty of             *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU          *
- * Lesser General Public License for more details.                            *
- *                                                                            *
- * You should have received a copy of the GNU Lesser General Public           *
- * License along with this library; if not, write to the Free Software        *
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  *
- *                                                                            *
- ******************************************************************************)
-
 (* Utils: utility functions for user convenience *)
 
-open Bigarray
 open Common
 open Read_ml
 open Write_ml
@@ -92,15 +67,13 @@ module type Make_binable_spec = sig
 
   type t
 
-  val to_binable : t -> Binable.binable
-  val of_binable : Binable.binable -> t
+  val to_binable : t -> Binable.t
+  val of_binable : Binable.t -> t
 end
 
 module Make_binable (S : Make_binable_spec) = struct
   module B = S.Binable
 
-  type binable = S.t
-
   let bin_size_t t = B.bin_size_t (S.to_binable t)
   let bin_write_t buf ~pos t = B.bin_write_t buf ~pos (S.to_binable t)
   let bin_write_t_ sptr eptr t = B.bin_write_t_ sptr eptr (S.to_binable t)
@@ -136,15 +109,13 @@ module type Make_binable1_spec = sig
 
   type 'a t
 
-  val to_binable : 'a t -> 'a Binable.binable
-  val of_binable : 'a Binable.binable -> 'a t
+  val to_binable : 'a t -> 'a Binable.t
+  val of_binable : 'a Binable.t -> 'a t
 end
 
 module Make_binable1 (S : Make_binable1_spec) = struct
   module B = S.Binable
 
-  type 'a binable = 'a S.t
-
   let bin_size_t bin_size_el t = B.bin_size_t bin_size_el (S.to_binable t)
 
   let bin_write_t bin_write_el buf ~pos t =
@@ -194,15 +165,13 @@ module type Make_binable2_spec = sig
 
   type ('a, 'b) t
 
-  val to_binable : ('a, 'b) t -> ('a, 'b) Binable.binable
-  val of_binable : ('a, 'b) Binable.binable -> ('a, 'b) t
+  val to_binable : ('a, 'b) t -> ('a, 'b) Binable.t
+  val of_binable : ('a, 'b) Binable.t -> ('a, 'b) t
 end
 
 module Make_binable2 (S : Make_binable2_spec) = struct
   module B = S.Binable
 
-  type ('a, 'b) binable = ('a, 'b) S.t
-
   let bin_size_t bin_size_el1 bin_size_el2 t =
     B.bin_size_t bin_size_el1 bin_size_el2 (S.to_binable t)
 
@@ -259,7 +228,7 @@ module type Make_iterable_binable_spec = sig
 
   val module_name : string option
   val length : t -> int
-  val iter : f : (el -> unit) -> t -> unit
+  val iter : t -> f : (el -> unit) -> unit
   val init : int -> acc
   val insert : acc -> el -> int -> acc
   val finish : acc -> t
@@ -271,8 +240,6 @@ end
 module Make_iterable_binable (S : Make_iterable_binable_spec) = struct
   open S
 
-  type binable = t
-
   let raise_concurrent_modification =
     match module_name with
     | None -> raise_concurrent_modification
@@ -338,13 +305,6 @@ module Make_iterable_binable (S : Make_iterable_binable_spec) = struct
       unsafe_write = bin_write_t_;
     }
 
-  let bin_writer_t =
-    {
-      size = bin_size_t;
-      write = bin_write_t;
-      unsafe_write = bin_write_t_;
-    }
-
   let bin_reader_t =
     {
       read = bin_read_t;
@@ -366,7 +326,7 @@ module type Make_iterable_binable1_spec = sig
 
   val module_name : string option
   val length : 'a t -> int
-  val iter : f : ('a el -> unit) -> 'a t -> unit
+  val iter : 'a t -> f : ('a el -> unit) -> unit
   val init : int -> 'a acc
   val insert : 'a acc -> 'a el -> int -> 'a acc
   val finish : 'a acc -> 'a t
@@ -378,8 +338,6 @@ end
 module Make_iterable_binable1 (S : Make_iterable_binable1_spec) = struct
   open S
 
-  type 'a binable = 'a t
-
   let raise_concurrent_modification =
     match module_name with
     | None -> raise_concurrent_modification
@@ -447,15 +405,6 @@ module Make_iterable_binable1 (S : Make_iterable_binable1_spec) = struct
         bin_write_t_ bin_writer.unsafe_write sptr eptr v);
     }
 
-  let bin_writer_t bin_writer =
-    {
-      size = (fun v -> bin_size_t bin_writer.size v);
-      write = (fun buf ~pos v ->
-        bin_write_t bin_writer.unsafe_write buf ~pos v);
-      unsafe_write = (fun sptr eptr v ->
-        bin_write_t_ bin_writer.unsafe_write sptr eptr v);
-    }
-
   let bin_reader_t bin_reader =
     {
       read = (fun buf ~pos_ref ->
@@ -479,7 +428,7 @@ module type Make_iterable_binable2_spec = sig
 
   val module_name : string option
   val length : ('a, 'b) t -> int
-  val iter : f : (('a, 'b) el -> unit) -> ('a, 'b) t -> unit
+  val iter : ('a, 'b) t -> f : (('a, 'b) el -> unit) -> unit
   val init : int -> ('a, 'b) acc
   val insert : ('a, 'b) acc -> ('a, 'b) el -> int -> ('a, 'b) acc
   val finish : ('a, 'b) acc -> ('a, 'b) t
@@ -491,8 +440,6 @@ end
 module Make_iterable_binable2 (S : Make_iterable_binable2_spec) = struct
   open S
 
-  type ('a, 'b) binable = ('a, 'b) t
-
   let raise_concurrent_modification =
     match module_name with
     | None -> raise_concurrent_modification
@@ -553,17 +500,6 @@ module Make_iterable_binable2 (S : Make_iterable_binable2_spec) = struct
   let bin_read_t__ _sptr_ptr _eptr _n =
     Unsafe_read_c.raise_variant_wrong_type "t"
 
-  let bin_writer_t bin_writer1 bin_writer2 =
-    {
-      size = (fun v -> bin_size_t bin_writer1.size bin_writer2.size v);
-      write = (fun buf ~pos v ->
-        bin_write_t
-          bin_writer1.unsafe_write bin_writer2.unsafe_write buf ~pos v);
-      unsafe_write = (fun sptr eptr v ->
-          bin_write_t_
-            bin_writer1.unsafe_write bin_writer2.unsafe_write sptr eptr v);
-    }
-
   let bin_writer_t bin_writer1 bin_writer2 =
     {
       size = (fun v -> bin_size_t bin_writer1.size bin_writer2.size v);
