@@ -1,39 +1,6 @@
-(******************************************************************************
- *                             Bin-prot                                       *
- *                                                                            *
- * Copyright (C) 2008- Jane Street Holding, LLC                               *
- *    Contact: opensource@janestreet.com                                      *
- *    WWW: http://www.janestreet.com/ocaml                                    *
- *    Author: Markus Mottl                                                    *
- *                                                                            *
- * This file is derived from file "pa_tywith.ml" of version 0.45 of the       *
- * library "Tywith".                                                          *
- *                                                                            *
- * Tywith is Copyright (C) 2004, 2005 by                                      *
- *                                                                            *
- *    Martin Sandin  <msandin@hotmail.com>                                    *
- *                                                                            *
- *                                                                            *
- * This library is free software; you can redistribute it and/or              *
- * modify it under the terms of the GNU Lesser General Public                 *
- * License as published by the Free Software Foundation; either               *
- * version 2 of the License, or (at your option) any later version.           *
- *                                                                            *
- * This library is distributed in the hope that it will be useful,            *
- * but WITHOUT ANY WARRANTY; without even the implied warranty of             *
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU          *
- * Lesser General Public License for more details.                            *
- *                                                                            *
- * You should have received a copy of the GNU Lesser General Public           *
- * License along with this library; if not, write to the Free Software        *
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  *
- *                                                                            *
- ******************************************************************************)
-
 (** Pa_bin_prot: Preprocessing Module for a Type Safe Binary Protocol *)
 
 open Printf
-open Lexing
 
 open Camlp4
 open PreCast
@@ -44,23 +11,17 @@ open Pa_type_conv
 
 let ( *** ) f g x = f (g x)
 
-let get_n_vars _loc name n =
-  let rec loop patts exprs n =
-    if n <= 0 then patts, exprs
-    else
-      let var = name ^ string_of_int n in
-      let patts = <:patt< $lid:var$ >> :: patts in
-      let exprs = <:expr< $lid:var$ >> :: exprs in
-      loop patts exprs (n - 1)
-  in
-  loop [] [] n
-
 let rec sig_of_tds cnv = function
   | TyDcl (_loc, type_name, tps, _rhs, _cl) -> cnv _loc type_name tps
   | TyAnd (_loc, tp1, tp2) ->
       <:sig_item< $sig_of_tds cnv tp1$; $sig_of_tds cnv tp2$ >>
   | _ -> assert false  (* impossible *)
 
+let mk_full_type loc type_name tps =
+  let coll_args tp param =
+    <:ctyp@loc< $tp$ $Gen.drop_variance_annotations param$ >>
+  in
+  List.fold_left coll_args <:ctyp@loc< $lid:type_name$ >> tps
 
 (* Generators for the binary protocol *)
 
@@ -93,9 +54,8 @@ module Sig_generate_writer = struct
       value $lid:"bin_writer_" ^ type_name$ : $bin_writer$
     >>
 
-  let mk_sig tds = <:sig_item< $sig_of_tds sig_of_td tds$ >>
-
-  let () = add_sig_generator "bin_write" mk_sig
+  let mk_sig _rec tds = <:sig_item< $sig_of_tds sig_of_td tds$ >>
+  let () = add_sig_generator ~delayed:true "bin_write" mk_sig;
 end
 
 
@@ -122,15 +82,14 @@ module Sig_generate_reader = struct
       loop <:ctyp< $lid:type_name$ >> tps
     in
     <:sig_item<
-      value $lid:"bin_read_" ^ type_name$: $bin_read$;
+      value $lid:"bin_read_" ^ type_name$ : $bin_read$;
       value $lid:"bin_read_" ^ type_name ^ "_"$ : $bin_read_$;
       value $lid:"bin_read_" ^ type_name ^ "__"$ : $bin_read__$;
       value $lid:"bin_reader_" ^ type_name$ : $bin_reader$
     >>
 
-  let mk_sig tds = <:sig_item< $sig_of_tds sig_of_td tds$ >>
-
-  let () = add_sig_generator "bin_read" mk_sig
+  let mk_sig _rec tds = <:sig_item< $sig_of_tds sig_of_td tds$ >>
+  let () = add_sig_generator ~delayed:true "bin_read" mk_sig;
 end
 
 
@@ -147,21 +106,15 @@ module Sig_generate_tp_class = struct
     let bin_tp_class = loop <:ctyp< $lid:type_name$ >> tps in
     <:sig_item< value $lid:"bin_" ^ type_name$ : $bin_tp_class$ >>
 
-  let mk_sig tds = <:sig_item< $sig_of_tds sig_of_td tds$ >>
-  let () = add_sig_generator "bin_type_class" mk_sig
+  let mk_sig _rec tds = <:sig_item< $sig_of_tds sig_of_td tds$ >>
+  let () = add_sig_generator ~delayed:true "bin_type_class" mk_sig
 end
 
 
 (* Generates the signature for binary protocol *)
 module Sig_generate = struct
   let () =
-    add_sig_generator "bin_io" (fun tds ->
-      let _loc = Loc.ghost in
-      <:sig_item<
-        $Sig_generate_writer.mk_sig tds$;
-        $Sig_generate_reader.mk_sig tds$;
-        $Sig_generate_tp_class.mk_sig tds$;
-      >>)
+    add_sig_set "bin_io" ~set:["bin_write"; "bin_read"; "bin_type_class"];
 end
 
 
@@ -181,7 +134,7 @@ module Generate_bin_size = struct
     | <:ctyp< $tp1$ $tp2$ >> ->
         `Fun (bin_size_appl_fun full_type_name _loc tp1 tp2)
     | <:ctyp< ( $tup:tp$ ) >> -> bin_size_tuple full_type_name _loc tp
-    | <:ctyp< '$parm$ >> -> `Fun (Gen.ide _loc ("_size_of_" ^ parm))
+    | <:ctyp< '$parm$ >> -> `Fun <:expr< $lid:"_size_of_" ^ parm$ >>
     | <:ctyp< $id:id$ >> -> `Fun (bin_size_path_fun _loc id)
     | <:ctyp< $_$ -> $_$ >> ->
         failwith
@@ -243,7 +196,7 @@ module Generate_bin_size = struct
   and bin_size_tuple full_type_name _loc tp =
     let cnv_patts patts = <:patt< ( $tup:paCom_of_list patts$ ) >> in
     let get_tp tp = tp in
-    let mk_patt _loc v_name _ = Gen.idp _loc v_name in
+    let mk_patt _loc v_name _ = <:patt< $lid:v_name$ >> in
     bin_size_tup_rec full_type_name _loc cnv_patts get_tp mk_patt tp
 
   (* Conversion of records *)
@@ -304,7 +257,7 @@ module Generate_bin_size = struct
     let bindings =
       let mk_binding parm =
         <:binding<
-          $Gen.idp _loc ("_size_of_" ^ parm)$ = fun _v ->
+          $lid:"_size_of_" ^ parm$ = fun _v ->
             raise (Bin_prot.Common.Poly_rec_write $str:full_type_name$)
         >>
       in
@@ -344,7 +297,7 @@ module Generate_bin_size = struct
       | <:ctyp< $uid:_$ >> -> has_atoms := true; <:match_case< >>
       | <:ctyp< $uid:cnstr$ of $tp$ >> ->
           let get_tp tp = tp in
-          let mk_patt _loc v_name _ = Gen.idp _loc v_name in
+          let mk_patt _loc v_name _ = <:patt< $lid:v_name$ >> in
           let patts, size_args =
             bin_size_args full_type_name _loc get_tp mk_patt tp
           in
@@ -357,7 +310,9 @@ module Generate_bin_size = struct
             let size = $size_tag$ in
             $size_args$
           >>
-      | _ -> failwith "branch_sum: unknown type"
+      | <:ctyp< $_$ : $_$ >> as tp -> Gen.error tp ~fn:"bin_size_sum"
+        ~msg:"GADTs are not supported by bin_prot"
+      | tp -> Gen.unknown_type tp "bin_size_sum"
     in
     let nonatom_matchings = loop alts in
     let matchings =
@@ -393,7 +348,8 @@ module Generate_bin_size = struct
       | `Match matchings -> <:expr< fun [ $matchings$ ] >>
     in
     let tparam_cnvs = List.map ((^) "_size_of_" *** Gen.get_tparam_id) tps in
-    let tparam_patts = List.map (Gen.idp _loc) tparam_cnvs in
+    let mk_pat id = <:patt< $lid:id$ >> in
+    let tparam_patts = List.map mk_pat tparam_cnvs in
     <:binding<
       $lid:"bin_size_" ^ type_name$ = $Gen.abstract _loc tparam_patts body$
     >>
@@ -404,22 +360,22 @@ module Generate_bin_size = struct
     | TyAnd (_loc, tp1, tp2) -> bin_size_tds (bin_size_tds acc tp2) tp1
     | _ -> assert false  (* impossible *)
 
-  let bin_size tds =
+  let bin_size rec_ tds =
     let bindings, recursive, _loc =
       match tds with
       | TyDcl (_loc, type_name, tps, rhs, _cl) ->
           let binding = bin_size_td _loc type_name tps rhs in
-          [binding], Gen.type_is_recursive type_name rhs, _loc
-      | TyAnd (_loc, _, _) -> bin_size_tds [] tds, true, _loc
+          [binding], rec_ && Gen.type_is_recursive type_name rhs, _loc
+      | TyAnd (_loc, _, _) -> bin_size_tds [] tds, rec_, _loc
       | _ -> assert false  (* impossible *)
     in
     if recursive then <:str_item< value rec $list:bindings$ >>
     else <:str_item< value $list:bindings$ >>
 end
 
-
 (* Generator for converters of OCaml-values to the binary protocol *)
 module Generate_bin_write = struct
+
   let mk_abst_call _loc tn rev_path =
     <:expr<
       $id:Gen.ident_of_rev_path _loc (("bin_write_" ^ tn ^ "_") :: rev_path)$
@@ -436,7 +392,7 @@ module Generate_bin_write = struct
     | <:ctyp< $tp1$ $tp2$ >> ->
         `Fun (bin_write_appl_fun full_type_name _loc tp1 tp2)
     | <:ctyp< ( $tup:tp$ ) >> -> bin_write_tuple full_type_name _loc tp
-    | <:ctyp< '$parm$ >> -> `Fun (Gen.ide _loc ("_write_" ^ parm))
+    | <:ctyp< '$parm$ >> -> `Fun <:expr< $lid:"_write_" ^ parm$ >>
     | <:ctyp< $id:id$ >> -> `Fun (bin_write_path_fun _loc id)
     | <:ctyp< $_$ -> $_$ >> ->
         failwith
@@ -496,7 +452,7 @@ module Generate_bin_write = struct
   and bin_write_tuple full_type_name _loc tp =
     let cnv_patts patts = <:patt< ( $tup:paCom_of_list patts$ ) >> in
     let get_tp tp = tp in
-    let mk_patt _loc v_name _ = Gen.idp _loc v_name in
+    let mk_patt _loc v_name _ = <:patt< $lid:v_name$ >> in
     bin_write_tup_rec full_type_name _loc cnv_patts get_tp mk_patt tp
 
   (* Conversion of records *)
@@ -557,7 +513,7 @@ module Generate_bin_write = struct
     let bindings =
       let mk_binding parm =
         <:binding<
-          $Gen.idp _loc ("_write_" ^ parm)$ = fun _sptr _eptr _v ->
+          $lid:"_write_" ^ parm$ = fun _sptr _eptr _v ->
             raise (Bin_prot.Common.Poly_rec_write $str:full_type_name$)
         >>
       in
@@ -602,7 +558,7 @@ module Generate_bin_write = struct
           i + 1, <:match_case< $uid:cnstr$ -> $write_tag$ $`int:i$ >>
       | <:ctyp< $uid:cnstr$ of $tp$ >> ->
           let get_tp tp = tp in
-          let mk_patt _loc v_name _ = Gen.idp _loc v_name in
+          let mk_patt _loc v_name _ = <:patt< $lid:v_name$ >> in
           let patts, write_args =
             bin_write_args full_type_name _loc get_tp mk_patt tp
           in
@@ -618,7 +574,9 @@ module Generate_bin_write = struct
             >>
           in
           i + 1, case
-      | _ -> failwith "branch_sum: unknown type"
+      | <:ctyp< $_$ : $_$ >> as tp -> Gen.error tp ~fn:"bin_write_sum"
+        ~msg:"GADTs are not supported by bin_prot"
+      | tp -> Gen.unknown_type tp "bin_write_sum"
     in
     `Match (snd (loop 0 alts))
 
@@ -648,7 +606,8 @@ module Generate_bin_write = struct
       | `Match matchings -> <:expr< fun sptr eptr -> fun [ $matchings$ ] >>
     in
     let tparam_cnvs = List.map ( (^) "_write_" *** Gen.get_tparam_id) tps in
-    let tparam_patts = List.map (Gen.idp _loc) tparam_cnvs in
+    let mk_pat id = <:patt< $lid:id$ >> in
+    let tparam_patts = List.map mk_pat tparam_cnvs in
     let int_call = "bin_write_" ^ type_name ^ "_" in
     let ext_fun =
       let ext_body =
@@ -658,7 +617,8 @@ module Generate_bin_write = struct
             <:expr< Bin_prot.Write_ml.$call$ buf ~pos v >>
         | _ ->
             let app_call =
-              let tparam_exprs = List.map (Gen.ide _loc) tparam_cnvs in
+              let mk_expr name = <:expr< $lid:name$ >> in
+              let tparam_exprs = List.map mk_expr tparam_cnvs in
               Gen.apply _loc <:expr< $lid:int_call$ >> tparam_exprs
             in
             <:expr<
@@ -747,7 +707,7 @@ module Generate_bin_write = struct
     | TyAnd (_loc, tp1, tp2) -> bin_write_tds (bin_write_tds acc tp2) tp1
     | _ -> assert false  (* impossible *)
 
-  let bin_write tds =
+  let bin_write rec_ tds =
     let internals, externals1, externals2, recursive, _loc =
       match tds with
       | TyDcl (_loc, type_name, tps, rhs, _cl) ->
@@ -755,12 +715,12 @@ module Generate_bin_write = struct
             bin_write_td _loc type_name tps rhs
           in
           [internal], [external1], [external2],
-          Gen.type_is_recursive type_name rhs, _loc
+          rec_ && Gen.type_is_recursive type_name rhs, _loc
       | TyAnd (_loc, _, _) ->
           let res = bin_write_tds [] tds in
           let internals, many_externals = List.split res in
           let externals1, externals2 = List.split many_externals in
-          internals, externals1, externals2, true, _loc
+          internals, externals1, externals2, rec_, _loc
       | _ -> assert false  (* impossible *)
     in
     let internals_item =
@@ -768,7 +728,7 @@ module Generate_bin_write = struct
       else <:str_item< value $list:internals$ >>
     in
     <:str_item<
-      $Generate_bin_size.bin_size tds$;
+      $Generate_bin_size.bin_size rec_ tds$;
       $internals_item$;
       value $list:externals1$;
       value $list:externals2$;
@@ -781,6 +741,7 @@ end
 
 (* Generator for converters of binary protocol to OCaml-values *)
 module Generate_bin_read = struct
+
   let mk_abst_call _loc tn ?(internal = false) rev_path =
     let tnp =
       let tnn = "bin_read_" ^ tn in
@@ -849,7 +810,7 @@ module Generate_bin_read = struct
         in
         expr
     | <:ctyp< ( $tup:tp$ ) >> -> bin_read_tuple full_type_name _loc tp
-    | <:ctyp< '$parm$ >> -> `Closed (Gen.ide _loc ("_of__" ^ parm))
+    | <:ctyp< '$parm$ >> -> `Closed <:expr< $lid:"_of__" ^ parm$ >>
     | <:ctyp< $id:id$ >> -> `Closed (bin_read_path_fun _loc id)
     | <:ctyp< $_$ -> $_$ >> -> failwith "bin_read_arrow: cannot convert functions"
     | <:ctyp< [< $row_fields$ ] >> | <:ctyp< [> $row_fields$ ] >>
@@ -996,7 +957,7 @@ module Generate_bin_read = struct
     let bindings =
       let mk_binding parm =
         <:binding<
-          $Gen.idp _loc ("_of__" ^ parm)$ =
+          $lid:"_of__" ^ parm$ =
             fun _sptr_ptr _eptr ->
               raise (
                 Bin_prot.Unsafe_read_c.Error (
@@ -1022,7 +983,9 @@ module Generate_bin_read = struct
           let bindings, args_expr = handle_arg_tp _loc full_type_name arg_tp in
           let rhs = <:expr< let $bindings$ in $uid:atom$ $args_expr$ >> in
           mi + 1, <:match_case< $`int:mi$ -> $rhs$ >>
-      | _ -> assert false  (* impossible *)
+      | <:ctyp< $_$ : $_$ >> as tp -> Gen.error tp ~fn:"bin_read_sum"
+        ~msg:"GADTs are not supported by bin_prot"
+      | tp -> Gen.unknown_type tp "bin_read_sum"
     in
     let n_alts, mcs = loop 0 alts in
     let read_fun =
@@ -1084,12 +1047,9 @@ module Generate_bin_read = struct
 
   (* Generate code from type definitions *)
 
-  let bin_read_td _loc type_name tps rhs =
+  let bin_read_td _loc rec_ type_name tps rhs =
     let full_type_name = sprintf "%s.%s" (get_conv_path ()) type_name in
-    let coll_args tp param =
-      <:ctyp< $tp$ $Gen.drop_variance_annotations param$ >>
-    in
-    let full_type = List.fold_left coll_args <:ctyp< $lid:type_name$ >> tps in
+    let full_type = mk_full_type _loc type_name tps in
     let is_alias_ref = ref false in
     let handle_alias _loc tp =
       is_alias_ref := true;
@@ -1109,7 +1069,7 @@ module Generate_bin_read = struct
       List.split (
         List.map (function tp ->
             let name = "_of__" ^ Gen.get_tparam_id tp in
-            Gen.idp _loc name, Gen.ide _loc name
+            <:patt< $lid:name$ >>, <:expr< $lid:name$ >>
           )
           tps)
     in
@@ -1128,7 +1088,7 @@ module Generate_bin_read = struct
 
     let variant_int_call =
       let maybe_poly_var_name = "bin_read_" ^ type_name ^ "__" in
-      let maybe_poly_var_expr = Gen.ide _loc maybe_poly_var_name in
+      let maybe_poly_var_expr = <:expr< $lid:maybe_poly_var_name$ >> in
       <:expr<
         let vint =
           Bin_prot.Unsafe_read_c.bin_read_variant_int sptr_ptr eptr
@@ -1179,21 +1139,23 @@ module Generate_bin_read = struct
       in
 
       let abst_call =
-        if !is_alias_ref then
+        if !is_alias_ref && rec_ then
+          (* this inlining is provoking captures in the generated code
+             when using nonrec, so disabling it *)
           match oc_body with
           | `Closed expr -> <:expr< $expr$ sptr_ptr eptr >>
           | `Open body -> body
         else if !is_variant_ref then variant_int_call
         else
           let abst_name = "bin_read_" ^ type_name ^ "_" in
-          let abst_expr = Gen.ide _loc abst_name in
+          let abst_expr = <:expr< $lid:abst_name$ >> in
           <:expr< $Gen.apply _loc abst_expr arg_exprs$ sptr_ptr eptr >>
       in
       let user_fun =
         let user_body =
           match abst_call with
           | <:expr< Bin_prot.Unsafe_read_c.$call$ sptr_ptr eptr >> ->
-              <:expr< Bin_prot.Read_ml.$call$ buf ~pos_ref >>
+              <:expr< ((Bin_prot.Read_ml.$call$ buf ~pos_ref) : $full_type$) >>
           | _ ->
               <:expr<
                 let pos = !pos_ref in
@@ -1214,7 +1176,7 @@ module Generate_bin_read = struct
                     let cur =
                       Bin_prot.Unsafe_common.dealloc_sptr_ptr buf sptr_ptr
                     in
-                    do { pos_ref.contents := cur; v }
+                    do { pos_ref.contents := cur; (v : $full_type$) }
               >>
         in
         Gen.abstract _loc arg_patts <:expr< fun buf ~pos_ref -> $user_body$ >>
@@ -1351,7 +1313,7 @@ module Generate_bin_read = struct
     in
     let reader_binding =
       <:binding< $lid:"bin_reader_" ^ type_name$ =
-        $Gen.abstract _loc tparam_reader_patts reader$
+        ($Gen.abstract _loc tparam_reader_patts reader$)
       >>
     in
     (
@@ -1365,46 +1327,46 @@ module Generate_bin_read = struct
       )
     )
 
-  let rec bin_read_tds acc = function
+  let rec bin_read_tds rec_ acc = function
     | TyDcl (_loc, type_name, tps, rhs, _cl) ->
-        bin_read_td _loc type_name tps rhs :: acc
-    | TyAnd (_loc, tp1, tp2) -> bin_read_tds (bin_read_tds acc tp2) tp1
+        bin_read_td _loc rec_ type_name tps rhs :: acc
+    | TyAnd (_loc, tp1, tp2) ->
+      bin_read_tds rec_ (bin_read_tds rec_ acc tp2) tp1
     | _ -> assert false  (* impossible *)
 
   (* Generate code from type definitions *)
-  let bin_read tds =
+  let bin_read rec_ tds =
     let res, recursive, _loc =
       match tds with
       | TyDcl (_loc, type_name, tps, rhs, _cl) ->
-          let res = bin_read_td _loc type_name tps rhs in
-          [res], Gen.type_is_recursive type_name rhs, _loc
-      | TyAnd (_loc, _, _) -> bin_read_tds [] tds, true, _loc
+          let res = bin_read_td _loc rec_ type_name tps rhs in
+          [res], rec_ && Gen.type_is_recursive type_name rhs, _loc
+      | TyAnd (_loc, _, _) -> bin_read_tds rec_ [] tds, rec_, _loc
       | _ -> assert false  (* impossible *)
     in
     let poly_abst, user_bindings_readers = List.split res in
     let user_bindings, readers = List.split user_bindings_readers in
-    if recursive then
-      (* Improve code locality *)
-      let cnv (maybe_poly_var_binding, abst_binding) =
-        <:binding< $maybe_poly_var_binding$ and $abst_binding$ >>
-      in
-      let internal_bindings = List.map cnv poly_abst in
-      <:str_item<
-        value rec $list:internal_bindings$;
-        value $list:user_bindings$;
-        value $list:readers$
-      >>
-    else
-      (* Improve code locality *)
-      let cnv (maybe_poly_var_binding, abst_binding) =
-        <:str_item< value $maybe_poly_var_binding$; value $abst_binding$ >>
-      in
-      let internal_items = List.map cnv poly_abst in
-      <:str_item<
-        $list:internal_items$;
-        value $list:user_bindings$;
-        value $list:readers$
-      >>
+    let internal_str_item =
+      if recursive then
+        (* Improve code locality *)
+        let cnv (maybe_poly_var_binding, abst_binding) =
+          <:binding< $maybe_poly_var_binding$ and $abst_binding$ >>
+        in
+        let internal_bindings = List.map cnv poly_abst in
+        <:str_item< value rec $list:internal_bindings$ >>
+      else
+        (* Improve code locality *)
+        let cnv (maybe_poly_var_binding, abst_binding) =
+          <:str_item< value $maybe_poly_var_binding$; value $abst_binding$ >>
+        in
+        let internal_items = List.map cnv poly_abst in
+        <:str_item< $list:internal_items$ >>
+    in
+    <:str_item<
+      $internal_str_item$;
+      value $list:user_bindings$;
+      value $list:readers$;
+    >>
 
   (* Add code generator to the set of known generators *)
   let () = add_generator "bin_read" bin_read
@@ -1417,7 +1379,8 @@ module Generate_tp_class = struct
     let tparam_cnvs =
       List.map (fun tp -> "bin_" ^  Gen.get_tparam_id tp) tps
     in
-    let tparam_patts = List.map (Gen.idp _loc) tparam_cnvs in
+    let mk_pat id = <:patt< $lid:id$ >> in
+    let tparam_patts = List.map mk_pat tparam_cnvs in
     let writer =
       let tparam_exprs =
         List.map (fun tp ->
@@ -1461,23 +1424,24 @@ module Generate_tp_class = struct
     | _ -> assert false  (* impossible *)
 
   (* Generate code from type definitions *)
-  let bin_tp_class tds =
+  let bin_tp_class _rec tds =
     let _loc = Loc.ghost in
-    <:str_item< value rec $list:bin_tp_class_tds [] tds$ >>
+    <:str_item< value $list:bin_tp_class_tds [] tds$ >>
 
   (* Add code generator to the set of known generators *)
   let () = add_generator "bin_type_class" bin_tp_class
 end
 
-(* Add "bin_read", "bin_write" and "bin_tp_class" as "bin_io" to the
+(* Add "bin_read", "bin_write" and "bin_type_class" as "bin_io" to the
    set of generators *)
-let () =
-  add_generator
-    "bin_io"
-    (fun tds ->
-      let _loc = Loc.ghost in
-      <:str_item<
-        $Generate_bin_write.bin_write tds$;
-        $Generate_bin_read.bin_read tds$;
-        $Generate_tp_class.bin_tp_class tds$;
-      >>)
+module Str_generate = struct
+  let () =
+    add_generator
+      "bin_io"
+      (fun rec_ tds ->
+        let _loc = Loc.ghost in
+        let bin_write  = Generate_bin_write.bin_write rec_ tds in
+        let bin_read   = Generate_bin_read.bin_read rec_ tds in
+        let type_class = Generate_tp_class.bin_tp_class rec_ tds in
+        <:str_item< $bin_write$; $bin_read$; $type_class$ >>)
+end
